@@ -12,25 +12,29 @@
 -behaviour(gen_server).
 
 %% External exports
--export([start_link/0, start/0]).
+-export([start_link/0, start/0, stop/0]).
 
 %% gen_server callbacks
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2]).
--compile(export_all).
-
 -include("../include/yaws_api.hrl").
 
+-export([new_session/1,new_session/2,
+         cookieval_to_opaque/1,
+         print_sessions/0,
+         replace_session/2,
+         delete_session/1]).
+         
 
 -define(TTL, (30 * 60)).  % 30 minutes
 
 -record(ysession,
-	{cookie,       %% the cookie assigned to the session
-	 to,           %% greg secs untill timeout death
-	 ttl,          %% default time to live
-	 starttime,    %% When calendar:local_time() did sess start
-	 opaque        %% any data the user supplies
-	}).
-	 
+        {cookie,       %% the cookie assigned to the session
+         to,           %% greg secs untill timeout death
+         ttl,          %% default time to live
+         starttime,    %% When calendar:local_time() did sess start
+         opaque        %% any data the user supplies
+        }).
+         
 
 
 
@@ -41,29 +45,29 @@
 %%%----------------------------------------------------------------------
 start_link() ->
     gen_server:start_link({local, yaws_session_server}, 
-			  yaws_session_server, [], []).
+                          yaws_session_server, [], []).
 start() ->
     gen_server:start({local, yaws_session_server}, 
-		     yaws_session_server, [], []).
+                     yaws_session_server, [], []).
 stop() ->
-    gen_server:call(?MODULE, stop).
+    gen_server:call(?MODULE, stop, infinity).
 
 
 %% will return a new cookie as a string
 new_session(Opaque) ->
-    gen_server:call(?MODULE, {new_session, Opaque, ?TTL}).
+    gen_server:call(?MODULE, {new_session, Opaque, ?TTL}, infinity).
 
 new_session(Opaque, TTL) ->
-    gen_server:call(?MODULE, {new_session, Opaque, TTL}).
+    gen_server:call(?MODULE, {new_session, Opaque, TTL}, infinity).
 
 cookieval_to_opaque(CookieString) ->
     case ets:lookup(?MODULE, CookieString) of
-	[Y] ->
-	    Y2 = Y#ysession{to = gnow() + Y#ysession.ttl},
-	    ets:insert(?MODULE, Y2),
-	    {ok, Y#ysession.opaque};
-	[] ->
-	    {error, no_session}
+        [Y] ->
+            Y2 = Y#ysession{to = gnow() + Y#ysession.ttl},
+            ets:insert(?MODULE, Y2),
+            {ok, Y#ysession.opaque};
+        [] ->
+            {error, no_session}
     end.
 
 
@@ -72,22 +76,22 @@ print_sessions() ->
     io:format("** ~p sessions active ~n~n", [length(Ss)]),
     N = gnow(),
     lists:foreach(fun(S) ->
-			  io:format("Cookie   ~p ~n", [S#ysession.cookie]),
-			  io:format("Start    ~p ~n", [S#ysession.starttime]),
-			  io:format("TTL      ~p secs~n", [S#ysession.to - N]),
-			  io:format("Opaque   ~p ~n~n~n", [S#ysession.opaque]),
-			  ok
-		  end, Ss).
+                          io:format("Cookie   ~p ~n", [S#ysession.cookie]),
+                          io:format("Start    ~p ~n", [S#ysession.starttime]),
+                          io:format("TTL      ~p secs~n", [S#ysession.to - N]),
+                          io:format("Opaque   ~p ~n~n~n", [S#ysession.opaque]),
+                          ok
+                  end, Ss).
 
 
 replace_session(Cookie, NewOpaque) ->
     case ets:lookup(?MODULE, Cookie) of
-	[Y] ->
-	    Y2 = Y#ysession{to = gnow() + Y#ysession.ttl,
-			    opaque = NewOpaque},
-	    ets:insert(?MODULE, Y2);
-	[] ->
-	    error
+        [Y] ->
+            Y2 = Y#ysession{to = gnow() + Y#ysession.ttl,
+                            opaque = NewOpaque},
+            ets:insert(?MODULE, Y2);
+        [] ->
+            error
     end.
 
 
@@ -110,9 +114,17 @@ init([]) ->
     {X,Y,Z} = seed(),
     random:seed(X, Y, Z),
     ets:new(?MODULE, [set, named_table, public, {keypos, 2}]),
+    start_long_timer(),
     {ok, undefined, to()}.
 
+%% timeout once every hour even if the server handles traffic all the time.
+start_long_timer() ->
+    erlang:send_after(long_to(), self(), long_timeout).
 
+long_to() ->
+    60 * 60 * 1000.
+
+%% timeout if the server is idle for more than 2 minutes.
 to() ->
     2 * 60 * 1000.  
 
@@ -120,11 +132,11 @@ to() ->
 %% pretty good seed, but non portable
 seed() ->
     case (catch list_to_binary(
-	   os:cmd("dd if=/dev/urandom ibs=12 count=1 2>/dev/null"))) of
-	<<X:32, Y:32, Z:32>> ->
-	    {X, Y, Z};
-	_ ->
-	    now()
+           os:cmd("dd if=/dev/urandom ibs=12 count=1 2>/dev/null"))) of
+        <<X:32, Y:32, Z:32>> ->
+            {X, Y, Z};
+        _ ->
+            now()
     end.
 
 
@@ -146,10 +158,10 @@ handle_call({new_session, Opaque, TTL}, _From, _State) ->
     TS = calendar:local_time(),
     C = atom_to_list(node()) ++ [$-|integer_to_list(N)],
     NS = #ysession{cookie = C,
-		   starttime = TS,
-		   opaque = Opaque,
-		   to = Now + TTL,
-		   ttl = TTL},
+                   starttime = TS,
+                   opaque = Opaque,
+                   to = Now + TTL,
+                   ttl = TTL},
     ets:insert(?MODULE, NS),
     {reply, C, undefined, to()};
 
@@ -171,7 +183,7 @@ report_timedout_sess(_S) ->
 %%          {stop, Reason, State}            (terminate/2 is called)
 %%----------------------------------------------------------------------
 handle_cast(_Msg, State) ->
-    {noreply, State}.
+    {noreply, State, to()}.
 
 %%----------------------------------------------------------------------
 %% Func: handle_info/2
@@ -181,6 +193,11 @@ handle_cast(_Msg, State) ->
 %%----------------------------------------------------------------------
 handle_info(timeout, _State) ->
     trav_ets(),
+    {noreply, undefined, to()};
+
+handle_info(long_timeout, _State) ->
+    trav_ets(),
+    start_long_timer(),
     {noreply, undefined, to()}.
 
 
@@ -204,19 +221,19 @@ trav_ets(_N, '$end_of_table') ->
     ok;
 trav_ets(N, Key) ->
     case ets:lookup(?MODULE, Key) of
-	[Y] ->
-	    if
-		Y#ysession.to > N ->
-		    trav_ets(N, ets:next(?MODULE, Key));
-		true ->
-		    report_timedout_sess(Y),
-		    Next = ets:next(?MODULE, Key),
-		    ets:delete(?MODULE, Key),
-		    trav_ets(N, Next)
-		    
-	    end;
-	[] ->
-	   trav_ets(N, ets:next(?MODULE, Key))
+        [Y] ->
+            if
+                Y#ysession.to > N ->
+                    trav_ets(N, ets:next(?MODULE, Key));
+                true ->
+                    report_timedout_sess(Y),
+                    Next = ets:next(?MODULE, Key),
+                    ets:delete(?MODULE, Key),
+                    trav_ets(N, Next)
+                    
+            end;
+        [] ->
+           trav_ets(N, ets:next(?MODULE, Key))
     end.
 
 gnow() ->
