@@ -3,6 +3,10 @@
 %%% Author  : Willem de Jong (w.a.de.jong@gmail.com).
 %%% Desc.   : Common SOAP code.
 %%%-------------------------------------------------------------------
+
+%%% modified (WdJ, May 2007): deal with imports in the WSDL.
+%%% modified (WdJ, August 2007): the WSDL can contain more than 1 schema
+
 -module(yaws_soap_lib).
 
 -export([initModel/1, initModel/2, 
@@ -28,10 +32,21 @@
 -include("../include/soap.hrl").
 -include("../include/erlsom.hrl").  % FIXME a better solution ?
 
+
+%%-define(dbg(X,Y), 
+%%        error_logger:info_msg("*dbg ~p(~p): " X,
+%%                              [?MODULE, ?LINE | Y])).
+-define(dbg(X,Y), true).
+
+
 -record(yaws_soap_config, {atts, xsd_path,  user_module, wsdl_file, add_files}).
 -record(xsd_file, {atts, name, prefix, import_specs}).
 -record(import_specs, {atts, namespace, prefix, location}).
-
+%% this comes from erlsom_compile. Maybe better to have an accessor function 
+%% in erlsom_lib?
+-record(schemaType, 
+        {elInfo, targetNamespace, elementFormDefault, attributeFormDefault, 
+        blockDefault, finalDefault, version, imports, elements}).
 -define(DefaultPrefix, "p").
 
 
@@ -133,7 +148,10 @@ call(#wsdl{operations = Operations, model = Model}, Operation, Port, Service, He
 		    Request = make_request_body(XmlMessage),
 		    HttpHeaders = [],
 		    HttpClientOptions = [],
-		    HttpRes = http_request(URL, SoapAction, Request, HttpClientOptions, HttpHeaders),
+                    ?dbg("+++ Request = ~p~n", [Request]),
+		    HttpRes = http_request(URL, SoapAction, Request, 
+                                           HttpClientOptions, HttpHeaders),
+                    ?dbg("+++ HttpRes = ~p~n", [HttpRes]),
 		    case HttpRes of
 			{ok, _Code, _ReturnHeaders, Body} ->
 			    parseMessage(Body, Model);
@@ -155,12 +173,12 @@ parseMessage(Message, #wsdl{model = Model}) ->
     parseMessage(Message, Model);
 %%
 parseMessage(Message, Model) ->
-    case erlsom:parse(Message, Model) of
+    case erlsom:scan(Message, Model) of
 	{ok, #'soap:Envelope'{'Body' = #'soap:Body'{choice = Body},
-			      'Header' = undefined}} ->
+			      'Header' = undefined}, _} ->
 	    {ok, undefined, Body};
 	{ok, #'soap:Envelope'{'Body' = #'soap:Body'{choice = Body},
-			      'Header' = #'soap:Header'{choice = Header}}} ->
+			      'Header' = #'soap:Header'{choice = Header}}, _} ->
 	    {ok, Header, Body};
 	{error, ErrorMessage} ->
 	    {error, {decoding, ErrorMessage}}
@@ -170,7 +188,8 @@ parseMessage(Message, Model) ->
 findOperation(_Operation, _Port, _Service, []) ->
     false;
 findOperation(Operation, Port, Service, 
-	       [Op = #operation{operation = Operation, port = Port, service = Service} | _]) ->
+              [Op = #operation{operation = Operation, 
+                               port = Port, service = Service} | _]) ->
     Op;
 findOperation(Operation, Port, Service, [#operation{} | Tail]) ->
     findOperation(Operation, Port, Service, Tail).
@@ -198,7 +217,7 @@ initModel(WsdlFile, Prefix) ->
 initModelFile(ConfigFile) ->
     {ok, ConfigSchema} = erlsom:compile_xsd(config_file_xsd()),
     %% read (parse) the config file
-    {ok, Config} = erlsom:parse_file(ConfigFile, ConfigSchema),
+    {ok, Config, _} = erlsom:scan_file(ConfigFile, ConfigSchema),
     #yaws_soap_config{xsd_path = XsdPath, 
 		      wsdl_file = Wsdl,
 		      add_files = AddFiles} = Config,
@@ -216,26 +235,78 @@ initModel2(WsdlFile, Prefix, Path, Import, AddFiles) ->
 					       {include_files, [IncludeWsdl]}]),
     %% add the xsd model (since xsd is also used in the wsdl)
     WsdlModel2 = erlsom:add_xsd_model(WsdlModel),
-    %% parse Wsdl
-    {ok, WsdlFileContent} = get_url_file(rmsp(WsdlFile)),
-    {ok, ParsedWsdl} = erlsom:parse(WsdlFileContent, WsdlModel2),
-    %% get the xsd elements from this model, and hand it over to erlsom_compile.
-    Xsd = getXsdFromWsdl(ParsedWsdl),
-    %% Ns = [#ns{prefix="xsd", uri="http://www.w3.org/2001/XMLSchema"}],
-    %% TODO: pass the right options here
     Options = makeOptions(Import),
-    {ok, Model} = erlsom_compile:compile_parsed_xsd(Xsd, [{prefix, Prefix} | Options]),
+    %% parse Wsdl
+    {Model, Operations} = parseWsdls([WsdlFile], Prefix, WsdlModel2, Options, {undefined, []}),
     %% TODO: add files as required
     %% now compile envelope.xsd, and add Model
     {ok, EnvelopeModel} = erlsom:compile_xsd_file(filename:join([Path, "envelope.xsd"]),
 						  [{prefix, "soap"}]),
     SoapModel = erlsom:add_model(EnvelopeModel, Model),
     SoapModel2 = addModels(AddFiles, SoapModel),
-    Ports = getPorts(ParsedWsdl),
-    Operations = getOperations(ParsedWsdl, Ports),
     #wsdl{operations = Operations, model = SoapModel2}.
 
 
+%%% --------------------------------------------------------------------
+%%% Parse a list of WSDLs and import (recursively)
+%%% Returns {Model, Operations}
+%%% --------------------------------------------------------------------
+parseWsdls([], _Prefix, _WsdlModel, _Options, Acc) ->
+  Acc;
+parseWsdls([WsdlFile | Tail], Prefix, WsdlModel, Options, {AccModel, AccOperations}) ->
+  {ok, WsdlFileContent} = get_url_file(rmsp(WsdlFile)),
+  {ok, ParsedWsdl, _} = erlsom:scan(WsdlFileContent, WsdlModel),
+  %% get the xsd elements from this model, and hand it over to erlsom_compile.
+  Xsds = getXsdsFromWsdl(ParsedWsdl),
+  %% Now we need to build a list: [{Namespace, Xsd, Prefix}, ...] for all the Xsds in the WSDL.
+  %% This list is used when a schema inlcudes one of the other schemas. The AXIS java2wsdl 
+  %% generates wsdls that depend on this feature. 
+  ImportList = makeImportList(Xsds, []),
+  %% TODO: pass the right options here
+  Model2 = addSchemas(Xsds, AccModel, Prefix, Options, ImportList),
+  Ports = getPorts(ParsedWsdl),
+  Operations = getOperations(ParsedWsdl, Ports),
+  Imports = getImports(ParsedWsdl),
+  Acc2 = {Model2, Operations ++ AccOperations},  
+  %% process imports (recursively, so that imports in the imported files are 
+  %% processed as well).
+  %% For the moment, the namespace is ignored on operations etc. 
+  %% this makes it a bit easier to deal with imported wsdl's.
+  Acc3 = parseWsdls(Imports, Prefix, WsdlModel, Options, Acc2),
+  parseWsdls(Tail, Prefix, WsdlModel, Options, Acc3).
+
+%%% --------------------------------------------------------------------
+%%% build a list: [{Namespace, Xsd}, ...] for all the Xsds in the WSDL.
+%%% This list is used when a schema inlcudes one of the other schemas. The AXIS java2wsdl 
+%%% generates wsdls that depend on this feature. 
+makeImportList([], Acc) ->
+  Acc;
+makeImportList([#schemaType{targetNamespace = Tns} = Xsd | Tail], Acc) ->
+  makeImportList(Tail, [{Tns, undefined, Xsd} | Acc]).
+  
+
+%%% --------------------------------------------------------------------
+%%% compile each of the schemas, and add it to the model.
+%%% Returns Model
+%%% (TODO: using the same prefix for all XSDS makes no sense)
+%%% --------------------------------------------------------------------
+addSchemas([], AccModel, _Prefix, _Options, _ImportList) ->
+  AccModel;
+addSchemas([Xsd| Tail], AccModel, Prefix, Options, ImportList) ->
+  Model2 = case Xsd of
+             undefined -> 
+               AccModel;
+             _ -> 
+               {ok, Model} = 
+                 erlsom_compile:compile_parsed_xsd(Xsd, 
+                                                   [{prefix, Prefix},
+                                                    {include_files, ImportList} |Options]),
+               case AccModel of 
+                 undefined -> Model;
+                 _ -> erlsom:add_model(AccModel, Model)
+               end
+           end,
+  addSchemas(Tail, Model2, Prefix, Options, ImportList).
 
 %%% --------------------------------------------------------------------
 %%% Get a file from an URL spec.
@@ -243,16 +314,7 @@ initModel2(WsdlFile, Prefix, Path, Import, AddFiles) ->
 get_url_file("http://"++_ = URL) ->
     case http:request(URL) of
 	{ok,{{_HTTP,200,_OK}, _Headers, Body}} -> 
-	    case http_uri:parse(URL) of
-		{_Method, _Host, _Port, _Path, _Qargs} ->
-		    %% Erlang/OTP =< R11B-1
-		    {ok, Body};
-		{_Method, _UserInfo, _Host, _Port, _Path, _Qargs} ->
-		    %% Erlang/OTP >= R11B-2
-		    {ok, Body};
-		_ -> 
-		    {error, "failed to parse URL: "++URL}
-	    end;
+	    {ok, Body};
 	{ok,{{_HTTP,RC,Emsg}, _Headers, _Body}} -> 
 	    error_logger:error_msg("~p: http-request got: ~p~n", [?MODULE, {RC, Emsg}]),
 	    {error, "failed to retrieve: "++URL};
@@ -261,6 +323,10 @@ get_url_file("http://"++_ = URL) ->
 	    {error, "failed to retrieve: "++URL}
     end;
 get_url_file("file://"++Fname) ->
+    {ok, Bin} = file:read_file(Fname),
+    {ok, binary_to_list(Bin)};
+%% added this, since this is what is used in many WSDLs (i.e.: just a filename).
+get_url_file(Fname) ->
     {ok, Bin} = file:read_file(Fname),
     {ok, binary_to_list(Bin)}.
 
@@ -301,8 +367,9 @@ inets_request(URL, SoapAction, Request, Options, Headers) ->
 ibrowse_request(URL, SoapAction, Request, Options, Headers) ->
     case start_ibrowse() of
         ok ->
-            NewHeaders = [{"SOAPAction", SoapAction} | Headers],
-            NewOptions = [{content_type, "text/xml; encoding=utf-8"} | Options],
+            NewHeaders = [{"Content-Type", "text/xml; charset=utf-8"}, {"SOAPAction", SoapAction} | Headers],
+            NewOptions = Options,
+                         %%[{content_type, "text/xml; encoding=utf-8"} | Options],
             case ibrowse:send_req(URL, NewHeaders, post, Request, NewOptions) of
                 {ok, Status, ResponseHeaders, ResponseBody} ->
                     {ok, list_to_integer(Status), ResponseHeaders, ResponseBody};
@@ -425,6 +492,10 @@ getTopLevelElements([#'wsdl:anyTopLevelOptionalElement'{choice = Tuple}| Tail],
     _ -> getTopLevelElements(Tail, Type, Acc)
   end.
 
+getImports(Definitions) ->
+  Imports = getTopLevelElements(Definitions, 'wsdl:tImport'),
+  lists:map(fun(Import) -> Import#'wsdl:tImport'.location end, Imports).
+
 %% returns [#operation{}] 
 getOperations(ParsedWsdl, Ports) ->
   Bindings = getTopLevelElements(ParsedWsdl, 'wsdl:tBinding'),
@@ -484,9 +555,11 @@ searchPorts(BindingName, [Port | Tail], Acc) ->
   end.
 
 
-getXsdFromWsdl(Definitions) ->
-    [#'wsdl:tTypes'{choice = [Xsd]}] = getTopLevelElements(Definitions, 'wsdl:tTypes'),
-    Xsd.
+getXsdsFromWsdl(Definitions) ->
+  case getTopLevelElements(Definitions, 'wsdl:tTypes') of
+    [#'wsdl:tTypes'{choice = Xsds}] -> Xsds;
+    [] -> undefined
+  end.
 
 config_file_xsd() ->
 "<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">"
