@@ -35,86 +35,93 @@ start_link() ->
 init([]) ->
 
     Sess = {yaws_session_server, {yaws_session_server, start_link, []},
-	       permanent, 5000, worker, [yaws_session_server]},
+            permanent, 5000, worker, [yaws_session_server]},
 
     YawsLog = {yaws_log, {yaws_log, start_link, []},
-	       permanent, 5000, worker, [yaws_log]},
+               permanent, 5000, worker, [yaws_log]},
 
-    YawsServArgs = [get_app_args()],
+    YawsServArgs = [_Env = get_app_args()],
     YawsServ = {yaws_server, {yaws_server, start_link, YawsServArgs},
-	       permanent, 5000, worker, [yaws_server]},
+                permanent, 5000, worker, [yaws_server]},
 
     YawsRSS = {yaws_rss, {yaws_rss, start_link, []},
-	       permanent, 5000, worker, [yaws_rss]},
+               permanent, 5000, worker, [yaws_rss]},
 
     YawsEventManager = {yaws_event_manager, 
-			{gen_event, start_link,[{local,yaws_event_manager}]},
-			permanent, 5000, worker, [gen_event]},
+                        {gen_event, start_link,[{local,yaws_event_manager}]},
+                        permanent, 5000, worker, [gen_event]},
 
-    {ok,{{one_for_all,0,300}, [YawsLog, YawsRSS, YawsServ, Sess, YawsEventManager]}}.
+    %% The idea behind this is if we're running in an embedded env, 
+    %% typically the supervisor above us wants to control the restarts.
+    %% 
+    %% If we're running standalone --heart can restart the entire node
+    %% If heart is not used, we die.
+    %% 0, 1 means that we never want supervisor restarts
+    {ok,{{one_for_all, 0, 1}, [YawsLog, YawsRSS, YawsServ, Sess, 
+                               YawsEventManager]}}.
 
 %%----------------------------------------------------------------------
 %%----------------------------------------------------------------------
 get_app_args() ->
     AS=init:get_arguments(),
     Debug = case application:get_env(yaws, debug) of
-		undefined ->
-		    member({yaws, ["debug"]}, AS);
-		{ok, Val}  ->
-		    Val
-	    end,
+                undefined ->
+                    member({yaws, ["debug"]}, AS);
+                {ok, Val}  ->
+                    Val
+            end,
     Trace = case application:get_env(yaws, trace) of
-		undefined ->
-		    case {member({yaws, ["trace", "http"]}, AS),
-			  member({yaws, ["trace", "traffic"]}, AS)} of
-			{true, _} ->
-			    {true, http};
-			{_, true} ->
-			    {true, traffic};
-			_ ->
-			    false
-		    end;
-		{ok, http} ->
-		    {true, http};
-		{ok, traffic} ->
-		    {true, traffic};
-		_ ->
-		    false
-	    end,
+                undefined ->
+                    case {member({yaws, ["trace", "http"]}, AS),
+                          member({yaws, ["trace", "traffic"]}, AS)} of
+                        {true, _} ->
+                            {true, http};
+                        {_, true} ->
+                            {true, traffic};
+                        _ ->
+                            false
+                    end;
+                {ok, http} ->
+                    {true, http};
+                {ok, traffic} ->
+                    {true, traffic};
+                _ ->
+                    false
+            end,
     TraceOutput = case application:get_env(yaws, traceoutput) of
-		undefined ->
-		    member({yaws, ["traceoutput"]}, AS);
-		{ok, Val3}  ->
-		    Val3
-	    end,
+                      undefined ->
+                          member({yaws, ["traceoutput"]}, AS);
+                      {ok, Val3}  ->
+                          Val3
+                  end,
     Conf = case application:get_env(yaws, conf) of
-	       undefined ->
-		   find_c(AS);
-	       {ok, File} ->
-		   {file, File}
-	   end,
+               undefined ->
+                   find_c(AS);
+               {ok, File} ->
+                   {file, File}
+           end,
     RunMod = case application:get_env(yaws, runmod) of
-		 undefined ->
-		     find_runmod(AS);
-		 {ok,Mod} ->
-		     {ok,Mod}
-	     end,
+                 undefined ->
+                     find_runmod(AS);
+                 {ok,Mod} ->
+                     {ok,Mod}
+             end,
     Embedded = case application:get_env(yaws, embedded) of
-		   undefined ->
-		       false;
-		   {ok, Emb} ->
-		       Emb
-	       end,
+                   undefined ->
+                       false;
+                   {ok, Emb} ->
+                       Emb
+               end,
     Id = case application:get_env(yaws, id) of
-		   undefined ->
-		       undefined;
-		   {ok, Id0} ->
-		       Id0
-	       end,
+             undefined ->
+                 "default";
+             {ok, Id0} ->
+                 Id0
+         end,
 
     #env{debug = Debug, trace = Trace, 
-	 traceoutput = TraceOutput, conf = Conf, 
-	 runmod = RunMod, embedded = Embedded, id = Id}.
+         traceoutput = TraceOutput, conf = Conf, 
+         runmod = RunMod, embedded = Embedded, id = Id}.
 
 %%----------------------------------------------------------------------
 %%----------------------------------------------------------------------
